package net.krakatoa.proxy.listener;

import de.dytanic.cloudnet.driver.CloudNetDriver;
import de.dytanic.cloudnet.driver.service.ServiceConfiguration;
import de.dytanic.cloudnet.driver.service.ServiceInfoSnapshot;
import de.dytanic.cloudnet.driver.service.ServiceTask;
import de.dytanic.cloudnet.ext.bridge.BridgeServiceProperty;
import de.dytanic.cloudnet.ext.bridge.player.ICloudPlayer;
import de.dytanic.cloudnet.ext.bridge.player.IPlayerManager;
import de.dytanic.cloudnet.ext.bridge.player.ServicePlayer;
import earth.krakatao.events.KraSocketClientEventInterface;
import earth.krakatao.protocol.KraSocketClientProtocolDest;
import earth.krakatao.protocol.KraSocketClientProtocolMessage;
import earth.krakatao.protocol.KraSocketClientProtocolStatus;
import java.util.Collection;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;
import net.krakatoa.proxy.ProxySystem;
import net.md_5.bungee.api.ProxyServer;
import net.md_5.bungee.api.chat.TextComponent;
import net.md_5.bungee.api.connection.ProxiedPlayer;
import org.java_websocket.handshake.ServerHandshake;

public class SocketClientMessageListener implements KraSocketClientEventInterface {

  private final HashMap<String, Collection<ServicePlayer>> playersAutoRestartServerSystem = new HashMap<>();
  private final IPlayerManager playerManager = CloudNetDriver.getInstance().getServicesRegistry()
      .getFirstService(IPlayerManager.class);

  @Override
  public void onOpen(ServerHandshake serverHandshake) {
    for (ProxiedPlayer proxiedPlayer : ProxyServer.getInstance().getPlayers()) {
      proxiedPlayer.sendMessage(new TextComponent("§cSocket §8» §aConnected"));
    }
  }

  @Override
  public void onMessage(KraSocketClientProtocolMessage kraProtocolMessage) {
    System.out.println(
        "kraProtocolMessage " + kraProtocolMessage.getCmdID() + " " + kraProtocolMessage.getArgs());

    System.out.println(
        "sendMessagesQueue size: " + ProxySystem.getInstance().getKraSocketClientProtocol()
            .getSendQueueMessages().size());

    if (kraProtocolMessage.getStatus() == KraSocketClientProtocolStatus.REPLY.getStatus()
        && ProxySystem.getInstance().getKraSocketClientProtocol().getSendQueueMessages()
        .containsKey(kraProtocolMessage.getCmdID())) {

      KraSocketClientProtocolMessage kraSocketClientProtocolMessage = ProxySystem.getInstance()
          .getKraSocketClientProtocol().getSendQueueMessages().get(kraProtocolMessage.getCmdID());

      if (kraSocketClientProtocolMessage.getMessageConsumer() != null) {
        ProxySystem.getInstance().getLogger()
            .info("accept " + kraProtocolMessage.getCmdID());
        kraSocketClientProtocolMessage.getMessageConsumer().accept(kraProtocolMessage);

        ProxySystem.getInstance().getKraSocketClientProtocol()
            .removeMessageFromSendQueue(kraProtocolMessage.getCmdID());
      }
    }

    switch (kraProtocolMessage.getCmdNumber()) {
      case 15:    // @Deprecated: test - mobile or voice socket connected
        ProxyServer.getInstance().getScheduler().runAsync(ProxySystem.getInstance(), () -> {
          System.out.println(kraProtocolMessage.getUuid());

          ProxiedPlayer proxiedPlayer = ProxySystem.getInstance().getProxy()
              .getPlayer(kraProtocolMessage.getUuid());

          if (proxiedPlayer != null) {
            if (kraProtocolMessage.getArgs().equals("1")) {
              proxiedPlayer.sendMessage(new TextComponent("§5Mobile §8» §cConnection lost!"));
            } else {
              proxiedPlayer.sendMessage(
                  new TextComponent("§bVoice §8» §cConnection lost!"));
            }
          }
        });
        break;
      case 33329: { // stop server - server auto restart
        String serverName = kraProtocolMessage.getArgs().toLowerCase();

        if (serverName.equals("lobby")) { // start new lobby
          if (CloudNetDriver.getInstance().getServiceTaskProvider().isServiceTaskPresent("Lobby")) {
            ServiceTask serviceTask = CloudNetDriver.getInstance().getServiceTaskProvider()
                .getServiceTask("Lobby");

            if (serviceTask != null) {
              ServiceInfoSnapshot serviceInfoSnapshot = ServiceConfiguration.builder(serviceTask)
                  .build().createNewService();

              if (serviceInfoSnapshot != null) {
                System.out.println("serviceTask " + serviceInfoSnapshot.getName());
                playersAutoRestartServerSystem.put(serviceInfoSnapshot.getName(), null);

                serviceInfoSnapshot.provider().start();
              }
            }
          }
        } else { // game server
          ServiceInfoSnapshot serviceInfoSnapshot = CloudNetDriver.getInstance()
              .getCloudServiceProvider()
              .getCloudServiceByName(serverName);

          if (serviceInfoSnapshot != null) {
            serviceInfoSnapshot.getProperty(BridgeServiceProperty.PLAYERS).ifPresent(players -> {
              players.forEach(player -> {
                ProxiedPlayer proxiedPlayer = ProxySystem.getInstance().getProxy()
                    .getPlayer(player.getUniqueId());
                proxiedPlayer.sendMessage(
                    new TextComponent("§4AutoRestart §8» §cRestarting server..."));
              });

              playersAutoRestartServerSystem.put(serverName, players);
            });

            // stop target server
            CloudNetDriver.getInstance().getCloudServiceProvider(serviceInfoSnapshot).stop();
          }
        }
        break;
      }
      case 59739: { // send players back to server - auto restart system
        String serverName = kraProtocolMessage.getArgs().toLowerCase();

        System.out.println("servername is " + kraProtocolMessage.getArgs());

        if (playersAutoRestartServerSystem.containsKey(
            kraProtocolMessage.getArgs())) { // close old lobbies
          Collection<ServiceInfoSnapshot> serviceInfoSnapshots = CloudNetDriver.getInstance()
              .getCloudServiceProvider()
              .getCloudServices("Lobby");

          if (serviceInfoSnapshots != null) {
            serviceInfoSnapshots.forEach(serviceInfoSnapshot -> {
              System.out.println("containsKey" + playersAutoRestartServerSystem.containsKey(
                  serviceInfoSnapshot.getName()));
              if (!playersAutoRestartServerSystem.containsKey(serviceInfoSnapshot.getName())) {
                serviceInfoSnapshot.provider().stop();
              }
            });
            playersAutoRestartServerSystem.remove(kraProtocolMessage.getArgs());
          }
        } else if (playersAutoRestartServerSystem.containsKey(serverName)) { // send players back
          playersAutoRestartServerSystem.get(serverName).forEach(servicePlayer -> {
            ICloudPlayer cloudPlayer = playerManager.getOnlinePlayer(servicePlayer.getUniqueId());

            if (cloudPlayer != null) {
              cloudPlayer.getPlayerExecutor().connect(serverName);

              ProxiedPlayer proxiedPlayer = ProxySystem.getInstance().getProxy()
                  .getPlayer(cloudPlayer.getUniqueId());

              proxiedPlayer.sendMessage(new TextComponent("§4AutoRestart §8» §aSwitching back"));
            }
          });
          playersAutoRestartServerSystem.remove(serverName);
        }
        break;
      }
      case 53874: { // test message - ack
        KraSocketClientProtocolMessage kraSocketClientProtocolMessage = new KraSocketClientProtocolMessage(
            KraSocketClientProtocolStatus.REPLY.getStatus(),
            kraProtocolMessage.getCmdID(), KraSocketClientProtocolDest.BACKEND.getStatus(),
            kraProtocolMessage.getUuid(),
            (short) 53874, "", null);

        ProxySystem.getInstance().getKraSocketClient().getSocketClient()
            .SendMessage(kraSocketClientProtocolMessage);

        ProxySystem.getInstance().getKraSocketClientProtocol()
            .removeMessageFromSendQueue(kraProtocolMessage.getCmdID());

        System.out.println(
            "cmdNumber 53874 answered back to uuid:" + kraProtocolMessage.getUuid()
                + " cmdIDByBackend:"
                + kraProtocolMessage.getCmdID());
      }
      default:

        break;
    }
  }

  @Override
  public void onClose(int code, String reason, boolean remote) {
    for (ProxiedPlayer proxiedPlayer : ProxyServer.getInstance().getPlayers()) {
      proxiedPlayer.sendMessage(
          new TextComponent(
              "§cSocket §8» §c" + (remote ? "Connection! Closed by remote peer" : reason)));
    }

    if (code != 1) {
      ProxyServer.getInstance().getScheduler().schedule(ProxySystem.getInstance(), () -> {
        for (ProxiedPlayer proxiedPlayer : ProxyServer.getInstance().getPlayers()) {
          proxiedPlayer.sendMessage(new TextComponent("§cSocket §8» §7Reconnecting..."));
        }

        ProxySystem.getInstance().getKraSocketClient().getSocketClient().Reconnect();
      }, 1L, TimeUnit.SECONDS);
    }
  }
}
